
#define  IXP_MICROC

#include "dl_system.h"
#include <hardware.h>
#include <intrinsic.c>
#include <libc.c>
#include <rtl.c>
#include <stdlib.h>
#include <ixptypes.h>
#include <string.h>
#include <ixp.h>
#include <ixp_lib.h>
#include <dl_meta.h>
#include <dl_buf.c>

#define NUM_WORDS_TX_MESSAGE	1
#define 	zero 	0
#define 	one		1

typedef struct
{
	dl_buf_handle_t		dlBufHandle;			// lw0
	unsigned int 		buffer_next		: 32;	// lw1
	unsigned int		buffer_size		: 16;	// lw2
	unsigned int 		offset			: 16;
	unsigned int		packet_size		: 16;	// lw3
	unsigned int		free_list_id	: 4;
	unsigned int		rx_stat			: 4;
	unsigned int 		header_type		: 8;
	unsigned int		input_port		: 16;	// lw4
	unsigned int		output_port		: 16;
} ring_data_t3;



void main()
{

////////////////////////////////////////////////////////////////////////////////
//
//	Signal and register definitions
//
////////////////////////////////////////////////////////////////////////////////

	SIGNAL 			sig_scr_get;				// signal for scratch get
	SIGNAL volatile	sig_next;					// signal for next ctx thread ordering
	SIGNAL			sig_scr_put;				// signal for scratch put
	SIGNAL_PAIR	sig_get_dram_header_pair;
	SIGNAL	volatile system_init_done;

	__declspec(gp_reg) 	int temp;				// general purpose use
	__declspec(gp_reg) 	int port;				// input port number
	__declspec(sram_write_reg) unsigned	int txreq;		// tx request message to put on scratch rings
	__declspec(gp_reg) 	int port_out;			// output port to forward packet to
	__declspec(sram_read_reg) ring_data_t3 rdata;

	__declspec(shared gp_reg) unsigned int counter_p0, counter_p1, counter_p2, counter_p3;

// This is a global signal that indicates that 
// system initialization is done

	__assign_relative_register((void *)&system_init_done, 10);

////////////////////////////////////////////////////////////////////////////////
//
//	Context common initializations
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//	Context specific initializations
//
////////////////////////////////////////////////////////////////////////////////


	if(ctx() == 0)
	{
		__wait_for_all(&system_init_done);
		counter_p0 = 0;
		counter_p1 = 0;
		counter_p2 = 0;
		counter_p3 = 0;

		signal_same_ME(__signal_number(&sig_next), 1);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 2);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 3);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 4);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 5);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 6);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 7);
		__implicit_write((void *)&sig_next);
	}
	else
	{__wait_for_all(&sig_next);}

	if(ctx() == 0)
	{
		signal_same_ME(__signal_number(&sig_next), 0);
		__implicit_write((void *)&sig_next);	
	}

	signal_same_ME(__signal_number(&sig_scr_put), __ctx());
	__implicit_write(&sig_scr_put);	

////////////////////////////////////////////////////////////////////////////////
//
//	Main loop
//
////////////////////////////////////////////////////////////////////////////////
	
	while(1)
	{

			/// START //////////////////////////////////
			
			/// Wait1 //////////////////////////////////

			__wait_for_all(&sig_next, &sig_scr_put);

			scratch_get_ring(&rdata, (__declspec(scratch) unsigned int *)(POS_RX_RING_OUT<<2), 5, sig_done, &sig_scr_get);

			/// Signal1 ////////////////////////////////

			signal_same_ME(__signal_number(&sig_next), (__ctx())+1 );//sig_next_val);
			__implicit_write((void *)&sig_next);

			/// Wait2 //////////////////////////////////

			__wait_for_all(&sig_scr_get);

			if(rdata.dlBufHandle.value != 0 )
			{	
				port = rdata.input_port & 0xF;
				port_out = port;
				
				txreq = (1<<31) | (port_out << 24) | ((unsigned int)(rdata.dlBufHandle.lw_offset) & 0xFFFFFF); 		

				if(port_out == 0) 
				{
					while(inp_state_test(inp_state_scr_ring6_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_0<<2), 1, sig_done, &sig_scr_put);
					counter_p0++;
					continue;
				}
				if(port_out == 1) 
				{	while(inp_state_test(inp_state_scr_ring7_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_1<<2), 1, sig_done, &sig_scr_put);
					counter_p1++;
					continue;
				}
				if(port_out == 2) 
				{
					while(inp_state_test(inp_state_scr_ring8_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_2<<2), 1, sig_done, &sig_scr_put);
					counter_p2++;
					continue;
				}
				if(port_out == 3) 
				{
					while(inp_state_test(inp_state_scr_ring9_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_3<<2), 1, sig_done, &sig_scr_put);
					counter_p3++;
					continue;
				}
			}
			else
			{
				signal_same_ME(__signal_number(&sig_scr_put), __ctx());
				__implicit_write(&sig_scr_put);
			}
	}
}
