////////////////////////////////////////////////////////////////////////////////
//	
//	COPYRIGHT 2003 BY RADISYS CORPORATION.  ALL RIGHTS RESERVED
//
////////////////////////////////////////////////////////////////////////////////
//	pkt_echo.uc
//
//	This file sources packets from the packet_rx[] microblock and forwards them
//	to the corresponding port transmit queue for the sphy_mphy4_tx[] microblock.
//
////////////////////////////////////////////////////////////////////////////////

#define  IXP_MICROC

#include "dl_system.h"
#include <hardware.h>
#include <intrinsic.c>
#include <libc.c>
#include <rtl.c>
#include <stdlib.h>
#include <ixptypes.h>
#include <string.h>
#include <ixp.h>
#include <ixp_lib.h>
#include <dl_meta.h>
#include <dl_buf.c>

#include "BSOB-V-C4-CAM-A03-DT-ACL1-10K-32.c"

#define NUM_WORDS_TX_MESSAGE	1

void main()
{

////////////////////////////////////////////////////////////////////////////////
//
//	Signal and register definitions
//
////////////////////////////////////////////////////////////////////////////////

	SIGNAL 			sig_scr_get;		// signal for scratch get
	SIGNAL volatile	sig_next;			// signal for next ctx thread ordering
	SIGNAL			sig_scr_put;		// signal for scratch put

	__declspec(gp_reg) 	int temp;				// general purpose use
	__declspec(gp_reg) 	int port;				// input port number
	__declspec(sram_write_reg) unsigned	int txreq;		// tx request message to put on scratch rings
	__declspec(gp_reg) 	unsigned int sig_next_val;		// value to write to same_me_signal register
	__declspec(gp_reg) 	int port_out;			// output port to forward packet to
	#define 	zero 	0
	#define 	one		1

typedef struct
{
	dl_buf_handle_t		dlBufHandle;			// lw0
	unsigned int 		buffer_next		: 32;	// lw1
	unsigned int		buffer_size		: 16;	// lw2
	unsigned int 		offset			: 16;
	unsigned int		packet_size		: 16;	// lw3
	unsigned int		free_list_id	: 4;
	unsigned int		rx_stat			: 4;
	unsigned int 		header_type		: 8;
	unsigned int		input_port		: 16;	// lw4
	unsigned int		output_port		: 16;
} ring_data_t3;

typedef struct 
{
	//QW0
	unsigned int dest_MAC_high32;
	unsigned int dest_MAC_low16   :16;
	unsigned int source_MAC_high16:16;
	//QW1
	unsigned int source_MAC_low32;
	unsigned int pdu			  :16;
	unsigned int version   		  :4;
	unsigned int header_len       :4;
	unsigned int tos	   		  :8;
	//QW2
	unsigned int total_len		  :16;
	unsigned int id				  :16;
	unsigned int flag			  :4;
	unsigned int frag_offset	  :12;
	unsigned int ttl			  :8;
	unsigned int protocol		  :8;
	//QW3
	unsigned int checksum		  :16;
//	unsigned int source_addr;
	unsigned int source_addr_high	:16;
	unsigned int source_addr_low	:16;
	unsigned int destination_addr_high:16;
	//QW4
	unsigned int destination_addr_low:16;
	unsigned int source_port      :16;  // UDP and TCP only
	unsigned int destination_port :16;  // UDP and TCP only
	unsigned int reserver_1		  :16;
	//QW5
	unsigned long long	reserver_2;
} ip_header;

	__declspec(shared gp_reg)	unsigned int count_p0;
	__declspec(shared gp_reg)	unsigned int count_p1;
	__declspec(shared gp_reg)	unsigned int count_p2;
	__declspec(shared gp_reg)	unsigned int count_p3;
	__declspec(shared gp_reg)	unsigned int count_empty;
	__declspec(shared gp_reg)	unsigned int debug_cnt;

	__declspec(sram_read_reg) ring_data_t3 rdata;
	__declspec(dram_read_reg) ip_header  header;

// This is a global signal that indicates that 
// system initialization is done

	SIGNAL_PAIR	sig_get_dram_header_pair;
	SIGNAL	volatile system_init_done;
	__assign_relative_register((void *)&system_init_done, 10);

////////////////////////////////////////////////////////////////////////////////
//
//	Context common initializations
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//	Context specific initializations
//
////////////////////////////////////////////////////////////////////////////////

	if(ctx() == 0)
	{	
		sig_next_val = 1;
		count_p0 = zero;
		count_p1 = zero;
		count_p2 = zero;
		count_p3 = zero;
		count_empty = zero;
		debug_cnt = zero;
	}
	else if(ctx() == 1)
		sig_next_val = 2;
	else if(ctx() == 2)
		sig_next_val = 3;
	else if(ctx() == 3)
		sig_next_val = 4;
	else if(ctx() == 4)
		sig_next_val = 5;
	else if(ctx() == 5)
		sig_next_val = 6;
	else if(ctx() == 6)
		sig_next_val = 7;
	else if(ctx() == 7)
		sig_next_val = 0;	


/////////////////////////////////////////////////////////////////////////////////

	if(ctx() == 0)
	{
		__wait_for_all(&system_init_done);
		signal_same_ME(__signal_number(&sig_next), 1);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 2);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 3);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 4);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 5);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 6);
		__implicit_write((void *)&sig_next);
		signal_same_ME(__signal_number(&sig_next), 7);
		__implicit_write((void *)&sig_next);
	}
	else
	{__wait_for_all(&sig_next);}

	if(ctx() == 0)
	{
		signal_same_ME(__signal_number(&sig_next), 0);
		__implicit_write((void *)&sig_next);	
	}

	signal_same_ME(__signal_number(&sig_scr_put), __ctx());
	__implicit_write(&sig_scr_put);	

////////////////////////////////////////////////////////////////////////////////
//
//	Main loop
//
////////////////////////////////////////////////////////////////////////////////
	
	while(1)
	{

			/// START //////////////////////////////////
			
			/// Wait1 //////////////////////////////////

			__wait_for_all(&sig_next, &sig_scr_put);

			scratch_get_ring(&rdata, (__declspec(scratch) unsigned int *)(POS_RX_RING_OUT<<2), 5, sig_done, &sig_scr_get);

			/// Signal1 ////////////////////////////////

			signal_same_ME(__signal_number(&sig_next), sig_next_val);
			__implicit_write((void *)&sig_next);

			/// Wait2 //////////////////////////////////

			__wait_for_all(&sig_scr_get);
//			__implicit_read(&rdata);
			if(rdata.dlBufHandle.value != 0 )
			{
				// Get the IP header
				dram_read(&header, (__declspec(dram) unsigned int *) (Dl_BufGetData(rdata.dlBufHandle) + rdata.offset ), 5, sig_done, &sig_get_dram_header_pair);
				__wait_for_all(&sig_get_dram_header_pair);

				// Packet Classification Section Start//
				packet_classification_pal(header.source_addr_high << 16 | header.source_addr_low, 
					header.destination_addr_high << 16 | header.destination_addr_low, header.source_port, 
					header.destination_port, header.protocol, header.checksum);				
				// Packet Classification Section End //		

				port = rdata.input_port & 0xF;

				port_out = port + 1;
				if(port_out == 3) port_out = 0;	
				
				txreq = (1<<31) | (port_out << 24) | ((unsigned int)(rdata.dlBufHandle.lw_offset) & 0xFFFFFF); 		

				if(port_out == 0) 
				{
					while(inp_state_test(inp_state_scr_ring6_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_0<<2), 1, sig_done, &sig_scr_put);
					count_p0++;
					continue;
				}
				if(port_out == 1) 
				{	while(inp_state_test(inp_state_scr_ring7_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_1<<2), 1, sig_done, &sig_scr_put);
					count_p1++;
					continue;
				}
				if(port_out == 2) 
				{
					while(inp_state_test(inp_state_scr_ring8_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_2<<2), 1, sig_done, &sig_scr_put);
					count_p2++;
					continue;
				}
				if(port_out == 3) 
				{
					while(inp_state_test(inp_state_scr_ring9_status))
						ctx_swap();
					scratch_put_ring(&txreq, (__declspec(scratch) unsigned int *)(PACKET_TX_SCR_RING_3<<2), 1, sig_done, &sig_scr_put);
					count_p3++;
					continue;
				}
			}
			else
			{
				signal_same_ME(__signal_number(&sig_scr_put), __ctx());
				__implicit_write(&sig_scr_put);
				count_empty++;
			}
	}
}
